@@ -43,13 +43,15 @@ var helpText = func(colors logger.Colors) string {
   --minify-syntax       Use equivalent but shorter syntax
   --outdir=...          The output directory (for multiple entry points)
   --outfile=...         The output file (for one entry point)
-  --platform=...        Platform target (browser | node, default browser)
+  --platform=...        Platform target (browser | node | neutral,
+                        default browser)
   --serve=...           Start a local HTTP server on this host:port for outputs
   --sourcemap           Emit a source map
   --splitting           Enable code splitting (currently only for esm)
   --summary             Print some helpful information at the end of a build
   --target=...          Environment target (e.g. es2017, chrome58, firefox57,
                         safari11, edge16, node10, default esnext)
+  --watch               Watch mode: rebuild on file system changes
 
 ` + colors.Bold + `Advanced options:` + colors.Default + `
   --banner=...              Text to be prepended to each output file
@@ -80,7 +82,7 @@ var helpText = func(colors logger.Colors) string {
   --tree-shaking=...        Set to "ignore-annotations" to work with packages
                             that have incorrect tree-shaking annotations
   --tsconfig=...            Use this tsconfig.json file instead of other ones
-  --version                 Print the current version and exit (` + esbuildVersion + `)
+  --version                 Print the current version (` + esbuildVersion + `) and exit
 
 ` + colors.Bold + `Examples:` + colors.Default + `
   ` + colors.Dim + `# Produces dist/entry_point.js and dist/entry_point.js.map` + colors.Default + `
