@@ -906,11 +906,21 @@ func TestRemoveUnusedPureCommentCalls(t *testing.T) {
 				let new_at_yes = /* @__PURE__ */ new foo(bar);
 				let new_at_no = /* @__PURE__ */ new foo(bar());
 
+				let nospace_at_yes = /*@__PURE__*/ foo(bar);
+				let nospace_at_no = /*@__PURE__*/ foo(bar());
+				let nospace_new_at_yes = /*@__PURE__*/ new foo(bar);
+				let nospace_new_at_no = /*@__PURE__*/ new foo(bar());
+
 				let num_yes = /* #__PURE__ */ foo(bar);
 				let num_no = /* #__PURE__ */ foo(bar());
 				let new_num_yes = /* #__PURE__ */ new foo(bar);
 				let new_num_no = /* #__PURE__ */ new foo(bar());
 
+				let nospace_num_yes = /*#__PURE__*/ foo(bar);
+				let nospace_num_no = /*#__PURE__*/ foo(bar());
+				let nospace_new_num_yes = /*#__PURE__*/ new foo(bar);
+				let nospace_new_num_no = /*#__PURE__*/ new foo(bar());
+
 				let dot_yes = /* @__PURE__ */ foo(sideEffect()).dot(bar);
 				let dot_no = /* @__PURE__ */ foo(sideEffect()).dot(bar());
 				let new_dot_yes = /* @__PURE__ */ new foo(sideEffect()).dot(bar);
@@ -1211,3 +1221,148 @@ func TestTreeShakingImportIdentifier(t *testing.T) {
 		},
 	})
 }
+
+func TestTreeShakingUnaryOperators(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				// These operators may have side effects
+				let keep;
+				+keep;
+				-keep;
+				~keep;
+				delete keep;
+				++keep;
+				--keep;
+				keep++;
+				keep--;
+
+				// These operators never have side effects
+				let REMOVE;
+				!REMOVE;
+				void REMOVE;
+				typeof REMOVE;
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestTreeShakingBinaryOperators(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				// These operators may have side effects
+				let keep, keep2;
+				keep + keep2;
+				keep - keep2;
+				keep * keep2;
+				keep / keep2;
+				keep % keep2;
+				keep ** keep2;
+				keep < keep2;
+				keep <= keep2;
+				keep > keep2;
+				keep >= keep2;
+				keep in keep2;
+				keep instanceof keep2;
+				keep << keep2;
+				keep >> keep2;
+				keep >>> keep2;
+				keep == keep2;
+				keep != keep2;
+				keep | keep2;
+				keep & keep2;
+				keep ^ keep2;
+				keep = keep2;
+				keep += keep2;
+				keep -= keep2;
+				keep *= keep2;
+				keep /= keep2;
+				keep %= keep2;
+				keep **= keep2;
+				keep <<= keep2;
+				keep >>= keep2;
+				keep >>>= keep2;
+				keep |= keep2;
+				keep &= keep2;
+				keep ^= keep2;
+				keep ??= keep2;
+				keep ||= keep2;
+				keep &&= keep2;
+
+				// These operators never have side effects
+				let REMOVE, REMOVE2;
+				REMOVE === REMOVE2;
+				REMOVE !== REMOVE2;
+				REMOVE, REMOVE2;
+				REMOVE ?? REMOVE2;
+				REMOVE || REMOVE2;
+				REMOVE && REMOVE2;
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestTreeShakingNoBundleESM(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function keep() {}
+				function unused() {}
+				keep()
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatESModule,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestTreeShakingNoBundleCJS(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function keep() {}
+				function unused() {}
+				keep()
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatCommonJS,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestTreeShakingNoBundleIIFE(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function keep() {}
+				function REMOVE() {}
+				keep()
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatIIFE,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
