@@ -99,7 +99,8 @@ func TestLowerExponentiationOperatorNoBundle(t *testing.T) {
 			UnsupportedJSFeatures: es(2015),
 			AbsOutputFile:         "/out.js",
 		},
-		expectedScanLog: "/entry.js: error: Big integer literals are not available in the configured target environment\n",
+		expectedScanLog: `entry.js: error: Big integer literals are not available in the configured target environment
+`,
 	})
 }
 
@@ -899,14 +900,14 @@ func TestLowerAsyncES5(t *testing.T) {
 			UnsupportedJSFeatures: es(5),
 			AbsOutputFile:         "/out.js",
 		},
-		expectedScanLog: `/arrow-1.js: error: Transforming async functions to the configured target environment is not supported yet
-/arrow-2.js: error: Transforming async functions to the configured target environment is not supported yet
-/export-def-1.js: error: Transforming async functions to the configured target environment is not supported yet
-/export-def-2.js: error: Transforming async functions to the configured target environment is not supported yet
-/fn-expr.js: error: Transforming async functions to the configured target environment is not supported yet
-/fn-stmt.js: error: Transforming async functions to the configured target environment is not supported yet
-/obj-method.js: error: Transforming async functions to the configured target environment is not supported yet
-/obj-method.js: error: Transforming object literal extensions to the configured target environment is not supported yet
+		expectedScanLog: `arrow-1.js: error: Transforming async functions to the configured target environment is not supported yet
+arrow-2.js: error: Transforming async functions to the configured target environment is not supported yet
+export-def-1.js: error: Transforming async functions to the configured target environment is not supported yet
+export-def-2.js: error: Transforming async functions to the configured target environment is not supported yet
+fn-expr.js: error: Transforming async functions to the configured target environment is not supported yet
+fn-stmt.js: error: Transforming async functions to the configured target environment is not supported yet
+obj-method.js: error: Transforming async functions to the configured target environment is not supported yet
+obj-method.js: error: Transforming object literal extensions to the configured target environment is not supported yet
 `,
 	})
 }
@@ -943,33 +944,6 @@ func TestLowerAsyncSuperES2016NoBundle(t *testing.T) {
 	})
 }
 
-func TestLowerClassFieldStrict2020NoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.js": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.js"},
-		options: config.Options{
-			UnsupportedJSFeatures: es(2020),
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestLowerClassField2020NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -994,32 +968,6 @@ func TestLowerClassField2020NoBundle(t *testing.T) {
 	})
 }
 
-func TestLowerClassFieldStrictNextNoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.js": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.js"},
-		options: config.Options{
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestLowerClassFieldNextNoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1043,33 +991,6 @@ func TestLowerClassFieldNextNoBundle(t *testing.T) {
 	})
 }
 
-func TestTSLowerClassFieldStrict2020NoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.ts": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.ts"},
-		options: config.Options{
-			UnsupportedJSFeatures: es(2020),
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestTSLowerClassField2020NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1094,32 +1015,6 @@ func TestTSLowerClassField2020NoBundle(t *testing.T) {
 	})
 }
 
-func TestTSLowerClassPrivateFieldStrictNextNoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.ts": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.ts"},
-		options: config.Options{
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestTSLowerClassPrivateFieldNextNoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1185,6 +1080,48 @@ func TestLowerClassFieldStrictTsconfigJson2020(t *testing.T) {
 	})
 }
 
+func TestTSLowerClassFieldStrictTsconfigJson2020(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import loose from './loose'
+				import strict from './strict'
+				console.log(loose, strict)
+			`,
+			"/loose/index.ts": `
+				export default class {
+					foo
+				}
+			`,
+			"/loose/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"useDefineForClassFields": false
+					}
+				}
+			`,
+			"/strict/index.ts": `
+				export default class {
+					foo
+				}
+			`,
+			"/strict/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"useDefineForClassFields": true
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			UnsupportedJSFeatures: es(2020),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
 func TestTSLowerObjectRest2017NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
