@@ -16,12 +16,13 @@ import (
 	"sync"
 	"testing"
 
+	"github.com/evanw/esbuild/internal/cache"
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/resolver"
-	"github.com/kylelemons/godebug/diff"
+	"github.com/evanw/esbuild/internal/test"
 )
 
 func es(version int) compat.JSFeature {
@@ -36,7 +37,7 @@ func assertEqual(t *testing.T, a interface{}, b interface{}) {
 		stringA := fmt.Sprintf("%v", a)
 		stringB := fmt.Sprintf("%v", b)
 		if strings.Contains(stringA, "\n") {
-			t.Fatal(diff.Diff(stringB, stringA))
+			t.Fatal(test.Diff(stringB, stringA))
 		} else {
 			t.Fatalf("%s != %s", a, b)
 		}
@@ -47,7 +48,7 @@ func assertLog(t *testing.T, msgs []logger.Msg, expected string) {
 	t.Helper()
 	text := ""
 	for _, msg := range msgs {
-		text += msg.String(logger.StderrOptions{}, logger.TerminalInfo{})
+		text += msg.String(logger.OutputOptions{}, logger.TerminalInfo{})
 	}
 	assertEqual(t, text, expected)
 }
@@ -90,8 +91,9 @@ func (s *suite) expectBundled(t *testing.T, args bundled) {
 			args.options.AbsOutputDir = path.Dir(args.options.AbsOutputFile)
 		}
 		log := logger.NewDeferLog()
-		resolver := resolver.NewResolver(fs, log, args.options)
-		bundle := ScanBundle(log, fs, resolver, args.entryPaths, args.options)
+		caches := cache.MakeCacheSet()
+		resolver := resolver.NewResolver(fs, log, caches, args.options)
+		bundle := ScanBundle(log, fs, resolver, caches, args.entryPaths, args.options)
 		msgs := log.Done()
 		assertLog(t, msgs, args.expectedScanLog)
 
