@@ -27,6 +27,7 @@ type Platform uint8
 const (
 	PlatformBrowser Platform = iota
 	PlatformNode
+	PlatformNeutral
 )
 
 type StrictOptions struct {
@@ -47,6 +48,7 @@ const (
 	SourceMapInline
 	SourceMapLinkedWithComment
 	SourceMapExternalWithoutComment
+	SourceMapInlineAndExternal
 )
 
 type Loader int
@@ -224,6 +226,10 @@ type Options struct {
 	Stdin *StdinInfo
 }
 
+func IsTreeShakingEnabled(mode Mode, outputFormat Format) bool {
+	return mode == ModeBundle || (mode == ModeConvertFormat && outputFormat == FormatIIFE)
+}
+
 type InjectedDefine struct {
 	Source logger.Source
 	Data   js_ast.E
@@ -312,13 +318,15 @@ type OnResolveArgs struct {
 	Path       string
 	Importer   logger.Path
 	ResolveDir string
+	PluginData interface{}
 }
 
 type OnResolveResult struct {
 	PluginName string
 
-	Path     logger.Path
-	External bool
+	Path       logger.Path
+	External   bool
+	PluginData interface{}
 
 	Msgs        []logger.Msg
 	ThrownError error
@@ -332,7 +340,8 @@ type OnLoad struct {
 }
 
 type OnLoadArgs struct {
-	Path logger.Path
+	Path       logger.Path
+	PluginData interface{}
 }
 
 type OnLoadResult struct {
@@ -341,6 +350,7 @@ type OnLoadResult struct {
 	Contents      *string
 	AbsResolveDir string
 	Loader        Loader
+	PluginData    interface{}
 
 	Msgs        []logger.Msg
 	ThrownError error
