@@ -22,11 +22,11 @@ func expectParseError(t *testing.T, contents string, expected string) {
 	t.Run(contents, func(t *testing.T) {
 		t.Helper()
 		log := logger.NewDeferLog()
-		Parse(log, test.SourceForTest(contents), config.Options{})
+		Parse(log, test.SourceForTest(contents), Options{})
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
-			text += msg.String(logger.StderrOptions{}, logger.TerminalInfo{})
+			text += msg.String(logger.OutputOptions{}, logger.TerminalInfo{})
 		}
 		test.AssertEqual(t, text, expected)
 	})
@@ -37,12 +37,15 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 	t.Run(name, func(t *testing.T) {
 		t.Helper()
 		log := logger.NewDeferLog()
-		tree := Parse(log, test.SourceForTest(contents), options)
+		tree := Parse(log, test.SourceForTest(contents), Options{
+			MangleSyntax:           options.MangleSyntax,
+			UnsupportedCSSFeatures: options.UnsupportedCSSFeatures,
+		})
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
 			if msg.Kind == logger.Error {
-				text += msg.String(logger.StderrOptions{}, logger.TerminalInfo{})
+				text += msg.String(logger.OutputOptions{}, logger.TerminalInfo{})
 			}
 		}
 		assertEqual(t, text, "")
@@ -577,6 +580,19 @@ func TestSelector(t *testing.T) {
 	expectPrinted(t, "a:b:c {}", "a:b:c {\n}\n")
 	expectPrinted(t, "a:b(:c) {}", "a:b(:c) {\n}\n")
 	expectPrinted(t, "a: b {}", "a: b {\n}\n")
+
+	expectPrinted(t, "#id {}", "#id {\n}\n")
+	expectPrinted(t, "#--0 {}", "#--0 {\n}\n")
+	expectPrinted(t, "#\\-0 {}", "#\\-0 {\n}\n")
+	expectPrinted(t, "#\\30 {}", "#\\30  {\n}\n")
+	expectPrinted(t, "div#id {}", "div#id {\n}\n")
+	expectPrinted(t, "div#--0 {}", "div#--0 {\n}\n")
+	expectPrinted(t, "div#\\-0 {}", "div#\\-0 {\n}\n")
+	expectPrinted(t, "div#\\30 {}", "div#\\30  {\n}\n")
+	expectParseError(t, "#0 {}", "<stdin>: warning: Unexpected \"#0\"\n")
+	expectParseError(t, "#-0 {}", "<stdin>: warning: Unexpected \"#-0\"\n")
+	expectParseError(t, "div#0 {}", "<stdin>: warning: Unexpected \"#0\"\n")
+	expectParseError(t, "div#-0 {}", "<stdin>: warning: Unexpected \"#-0\"\n")
 }
 
 func TestNestedSelector(t *testing.T) {
