@@ -3,7 +3,6 @@ package css_printer
 import (
 	"testing"
 
-	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/css_parser"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/test"
@@ -21,12 +20,12 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 	t.Run(name, func(t *testing.T) {
 		t.Helper()
 		log := logger.NewDeferLog()
-		tree := css_parser.Parse(log, test.SourceForTest(contents), config.Options{})
+		tree := css_parser.Parse(log, test.SourceForTest(contents), css_parser.Options{})
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
 			if msg.Kind == logger.Error {
-				text += msg.String(logger.StderrOptions{}, logger.TerminalInfo{})
+				text += msg.String(logger.OutputOptions{}, logger.TerminalInfo{})
 			}
 		}
 		assertEqual(t, text, "")
