@@ -83,6 +83,16 @@ func (op OpCode) BinaryAssignTarget() AssignTarget {
 	return AssignTargetNone
 }
 
+func (op OpCode) IsShortCircuit() bool {
+	switch op {
+	case BinOpLogicalOr, BinOpLogicalOrAssign,
+		BinOpLogicalAnd, BinOpLogicalAndAssign,
+		BinOpNullishCoalescing, BinOpNullishCoalescingAssign:
+		return true
+	}
+	return false
+}
+
 type AssignTarget uint8
 
 const (
@@ -381,11 +391,6 @@ type EBinary struct {
 	Left  Expr
 	Right Expr
 	Op    OpCode
-
-	// If true, this expression was originally a self-assignment of the form
-	// "x = y". This is relevant because it's used to emit a warning later on
-	// about code that accidentally does "x = x".
-	WasIdentifierAssign bool
 }
 
 type EBoolean struct{ Value bool }
@@ -484,10 +489,9 @@ type EArrow struct {
 	Args []Arg
 	Body FnBody
 
-	IsAsync         bool
-	HasRestArg      bool
-	IsParenthesized bool
-	PreferExpr      bool // Use shorthand if true and "Body" is a single return statement
+	IsAsync    bool
+	HasRestArg bool
+	PreferExpr bool // Use shorthand if true and "Body" is a single return statement
 }
 
 type EFunction struct{ Fn Fn }
@@ -507,10 +511,6 @@ type EIdentifier struct {
 	// unwrapped if the resulting value is unused. Unwrapping means discarding
 	// the call target but keeping any arguments with side effects.
 	CallCanBeUnwrappedIfUnused bool
-
-	// If true, this identifier was originally a "x as T" or "<T>x" expression.
-	// This is used to avoid warnings about "x = x as T" assignments.
-	WasTypeScriptCast bool
 }
 
 // This is similar to an EIdentifier but it represents a reference to an ES6
@@ -664,35 +664,185 @@ func AssignStmt(a Expr, b Expr) Stmt {
 	return Stmt{Loc: a.Loc, Data: &SExpr{Value: Assign(a, b)}}
 }
 
-func Not(a Expr) Expr {
-	// "!!!a" => "!a"
-	if not, ok := a.Data.(*EUnary); ok && not.Op == UnOpNot && IsBooleanValue(not.Value) {
-		return not.Value
+// Wraps the provided expression in the "!" prefix operator. The expression
+// will potentially be simplified to avoid generating unnecessary extra "!"
+// operators. For example, calling this with "!!x" will return "!x" instead
+// of returning "!!!x".
+func Not(expr Expr) Expr {
+	if result, ok := MaybeSimplifyNot(expr); ok {
+		return result
 	}
-	return Expr{Loc: a.Loc, Data: &EUnary{UnOpNot, a}}
+	return Expr{Loc: expr.Loc, Data: &EUnary{Op: UnOpNot, Value: expr}}
+}
+
+// The given "expr" argument should be the operand of a "!" prefix operator
+// (i.e. the "x" in "!x"). This returns a simplified expression for the
+// whole operator (i.e. the "!x") if it can be simplified, or false if not.
+// It's separate from "Not()" above to avoid allocation on failure in case
+// that is undesired.
+func MaybeSimplifyNot(expr Expr) (Expr, bool) {
+	switch e := expr.Data.(type) {
+	case *EUnary:
+		// "!!!a" => "!a"
+		if e.Op == UnOpNot && IsBooleanValue(e.Value) {
+			return e.Value, true
+		}
+
+	case *EBinary:
+		// Make sure that these transformations are all safe for special values.
+		// For example, "!(a < b)" is not the same as "a >= b" if a and/or b are
+		// NaN (or undefined, or null, or possibly other problem cases too).
+		switch e.Op {
+		case BinOpLooseEq:
+			// "!(a == b)" => "a != b"
+			e.Op = BinOpLooseNe
+			return expr, true
+
+		case BinOpLooseNe:
+			// "!(a != b)" => "a == b"
+			e.Op = BinOpLooseEq
+			return expr, true
+
+		case BinOpStrictEq:
+			// "!(a === b)" => "a !== b"
+			e.Op = BinOpStrictNe
+			return expr, true
+
+		case BinOpStrictNe:
+			// "!(a !== b)" => "a === b"
+			e.Op = BinOpStrictEq
+			return expr, true
+
+		case BinOpComma:
+			// "!(a, b)" => "a, !b"
+			e.Right = Not(e.Right)
+			return expr, true
+		}
+	}
+
+	return Expr{}, false
 }
 
 func IsBooleanValue(a Expr) bool {
 	switch e := a.Data.(type) {
 	case *EBoolean:
 		return true
+
+	case *EIf:
+		return IsBooleanValue(e.Yes) && IsBooleanValue(e.No)
+
 	case *EUnary:
 		return e.Op == UnOpNot || e.Op == UnOpDelete
+
 	case *EBinary:
 		switch e.Op {
 		case BinOpStrictEq, BinOpStrictNe, BinOpLooseEq, BinOpLooseNe,
 			BinOpLt, BinOpGt, BinOpLe, BinOpGe,
 			BinOpInstanceof, BinOpIn:
 			return true
+
 		case BinOpLogicalOr, BinOpLogicalAnd:
 			return IsBooleanValue(e.Left) && IsBooleanValue(e.Right)
+
 		case BinOpNullishCoalescing:
 			return IsBooleanValue(e.Left)
 		}
 	}
+
 	return false
 }
 
+func IsNumericValue(a Expr) bool {
+	switch e := a.Data.(type) {
+	case *ENumber:
+		return true
+
+	case *EIf:
+		return IsNumericValue(e.Yes) && IsNumericValue(e.No)
+
+	case *EUnary:
+		switch e.Op {
+		case UnOpPos, UnOpNeg, UnOpCpl, UnOpPreDec, UnOpPreInc, UnOpPostDec, UnOpPostInc:
+			return true
+		}
+
+	case *EBinary:
+		switch e.Op {
+		case BinOpAdd:
+			return IsNumericValue(e.Left) && IsNumericValue(e.Right)
+
+		case BinOpSub, BinOpMul, BinOpDiv, BinOpRem,
+			BinOpBitwiseAnd, BinOpBitwiseOr, BinOpBitwiseXor,
+			BinOpShl, BinOpShr, BinOpUShr:
+			return true
+
+		case BinOpSubAssign, BinOpMulAssign, BinOpDivAssign, BinOpRemAssign,
+			BinOpBitwiseAndAssign, BinOpBitwiseOrAssign, BinOpBitwiseXorAssign,
+			BinOpShlAssign, BinOpShrAssign, BinOpUShrAssign:
+
+		case BinOpAssign, BinOpComma:
+			return IsNumericValue(e.Right)
+		}
+	}
+
+	return false
+}
+
+func IsStringValue(a Expr) bool {
+	switch e := a.Data.(type) {
+	case *EString:
+		return true
+
+	case *ETemplate:
+		return e.Tag == nil
+
+	case *EIf:
+		return IsStringValue(e.Yes) && IsStringValue(e.No)
+
+	case *EUnary:
+		return e.Op == UnOpTypeof
+
+	case *EBinary:
+		switch e.Op {
+		case BinOpAdd:
+			return IsStringValue(e.Left) || IsStringValue(e.Right)
+
+		case BinOpAssign, BinOpAddAssign, BinOpComma:
+			return IsNumericValue(e.Right)
+		}
+	}
+
+	return false
+}
+
+// The goal of this function is to "rotate" the AST if it's possible to use the
+// left-associative property of the operator to avoid unnecessary parentheses.
+//
+// When using this, make absolutely sure that the operator is actually
+// associative. For example, the "-" operator is not associative for
+// floating-point numbers.
+func JoinWithLeftAssociativeOp(op OpCode, a Expr, b Expr) Expr {
+	// "(a, b) op c" => "a, b op c"
+	if comma, ok := a.Data.(*EBinary); ok && comma.Op == BinOpComma {
+		comma.Right = JoinWithLeftAssociativeOp(op, comma.Right, b)
+		return a
+	}
+
+	// "a op (b op c)" => "(a op b) op c"
+	// "a op (b op (c op d))" => "((a op b) op c) op d"
+	if binary, ok := b.Data.(*EBinary); ok && binary.Op == op {
+		return JoinWithLeftAssociativeOp(
+			op,
+			JoinWithLeftAssociativeOp(op, a, binary.Left),
+			binary.Right,
+		)
+	}
+
+	// "a op b" => "a op b"
+	// "(a op b) op c" => "(a op b) op c"
+	return Expr{Loc: a.Loc, Data: &EBinary{Op: op, Left: a, Right: b}}
+}
+
 func JoinWithComma(a Expr, b Expr) Expr {
 	return Expr{Loc: a.Loc, Data: &EBinary{Op: BinOpComma, Left: a, Right: b}}
 }
