@@ -133,6 +133,7 @@ func TestTSTypes(t *testing.T) {
 	expectPrintedTS(t, "let x: {['x']: number}", "let x;\n")
 	expectPrintedTS(t, "let x: {['x'](): void}", "let x;\n")
 	expectPrintedTS(t, "let x: {[key: string]: number}", "let x;\n")
+	expectPrintedTS(t, "let x: () => void = Foo", "let x = Foo;\n")
 	expectPrintedTS(t, "let x: new () => void = Foo", "let x = Foo;\n")
 	expectPrintedTS(t, "let x = 'x' as keyof T", "let x = \"x\";\n")
 	expectPrintedTS(t, "let x = [1] as readonly [number]", "let x = [1];\n")
@@ -194,6 +195,14 @@ func TestTSTypes(t *testing.T) {
 
 	// TypeScript 4.1
 	expectPrintedTS(t, "let foo: `${'a' | 'b'}-${'c' | 'd'}` = 'a-c'", "let foo = \"a-c\";\n")
+
+	// TypeScript 4.2
+	expectPrintedTS(t, "let x: abstract new () => void = Foo", "let x = Foo;\n")
+	expectPrintedTS(t, "let x: abstract new <T>() => Foo<T>", "let x;\n")
+	expectPrintedTS(t, "let x: abstract new <T extends object>() => Foo<T>", "let x;\n")
+	expectParseErrorTS(t, "let x: abstract () => void = Foo", "<stdin>: error: Expected \";\" but found \"(\"\n")
+	expectParseErrorTS(t, "let x: abstract <T>() => Foo<T>", "<stdin>: error: Expected \";\" but found \"(\"\n")
+	expectParseErrorTS(t, "let x: abstract <T extends object>() => Foo<T>", "<stdin>: error: Expected \"?\" but found \">\"\n")
 }
 
 func TestTSAsCast(t *testing.T) {
@@ -1274,6 +1283,12 @@ func TestTSNew(t *testing.T) {
 	expectParseError(t, "new Foo!()", "<stdin>: error: Unexpected \"!\"\n")
 }
 
+func TestTSExponentiation(t *testing.T) {
+	// More info: https://github.com/microsoft/TypeScript/issues/41755
+	expectParseErrorTS(t, "await x! ** 2", "<stdin>: error: Unexpected \"**\"\n")
+	expectPrintedTS(t, "await x as any ** 2", "(await x) ** 2;\n")
+}
+
 func TestTSImport(t *testing.T) {
 	expectPrintedTS(t, "import {x} from 'foo'", "")
 	expectPrintedTS(t, "import {x} from 'foo'; log(x)", "import {x} from \"foo\";\nlog(x);\n")
@@ -1510,17 +1525,3 @@ h();
 Foo[_b] = 1;
 `)
 }
-
-func TestWarningReassign(t *testing.T) {
-	expectParseErrorTS(t, "let x; x = x", "<stdin>: warning: Assignment of \"x\" to itself has no effect\n")
-	expectParseErrorTS(t, "let x; x = (x)", "<stdin>: warning: Assignment of \"x\" to itself has no effect\n")
-	expectParseErrorTS(t, "let x; x = (true ? x : x)", "")
-	expectParseErrorTS(t, "let x; x = (false ? x : x)", "")
-	expectParseErrorTS(t, "let x; x = (0, x)", "")
-	expectParseErrorTS(t, "let x; (x as any) = x", "")
-	expectParseErrorTS(t, "let x; x = x as any", "")
-	expectParseErrorTS(t, "let x; (<any>x) = x", "")
-	expectParseErrorTS(t, "let x; x = <any>x", "")
-	expectParseErrorTS(t, "let x; x! = x", "")
-	expectParseErrorTS(t, "let x; x = x!", "")
-}
