@@ -86,59 +86,6 @@ func AppendSourceMapChunk(j *Joiner, prevEndState SourceMapState, startState Sou
 	j.AddBytes(sourceMap)
 }
 
-// Rewrite the source map to remove everything before "offset" and have the
-// generated position start from (0, 0) at that point. This is used when
-// erasing the variable declaration keyword from the start of a file.
-func RemovePrefixFromSourceMapChunk(buffer []byte, offset int) []byte {
-	// Avoid an out-of-bounds array access if the offset is 0. This doesn't
-	// happen normally but can happen if a source is remapped from another
-	// source map that is missing mappings. This is the case with some file
-	// in the "ant-design" project.
-	if offset == 0 {
-		return buffer
-	}
-
-	state := SourceMapState{}
-	i := 0
-
-	// Accumulate mappings before the break point
-	for i < offset {
-		if buffer[i] == ';' {
-			i++
-			continue
-		}
-
-		var si, ol, oc int
-		_, i = sourcemap.DecodeVLQ(buffer, i)
-		si, i = sourcemap.DecodeVLQ(buffer, i)
-		ol, i = sourcemap.DecodeVLQ(buffer, i)
-		oc, i = sourcemap.DecodeVLQ(buffer, i)
-		state.SourceIndex += si
-		state.OriginalLine += ol
-		state.OriginalColumn += oc
-
-		if i < len(buffer) && buffer[i] == ',' {
-			i++
-		}
-	}
-
-	// Rewrite the mapping at the break point
-	var si, ol, oc int
-	_, i = sourcemap.DecodeVLQ(buffer, i)
-	si, i = sourcemap.DecodeVLQ(buffer, i)
-	ol, i = sourcemap.DecodeVLQ(buffer, i)
-	oc, i = sourcemap.DecodeVLQ(buffer, i)
-	state.SourceIndex += si
-	state.OriginalLine += ol
-	state.OriginalColumn += oc
-
-	// Splice it in front of the buffer assuming it's big enough
-	first := appendMapping(nil, 0, SourceMapState{}, state)
-	buffer = buffer[i-len(first):]
-	copy(buffer, first)
-	return buffer
-}
-
 func appendMapping(buffer []byte, lastByte byte, prevState SourceMapState, currentState SourceMapState) []byte {
 	// Put commas in between mappings
 	if lastByte != 0 && lastByte != ';' && lastByte != '"' {
@@ -532,12 +479,6 @@ type printer struct {
 	hasPrevState        bool
 	lineOffsetTables    []LineOffsetTable
 
-	// For splicing adjacent files together
-	finalLocalSemi           int
-	firstDeclByteOffset      uint32
-	firstDeclSourceMapOffset uint32
-	hasDecl                  bool
-
 	// This is a workaround for a bug in the popular "source-map" library:
 	// https://github.com/mozilla/source-map/issues/261. The library will
 	// sometimes return null when querying a source map unless every line
@@ -868,27 +809,16 @@ func (p *printer) printIdentifierUTF16(name []uint16) {
 	}
 }
 
-func (p *printer) addSourceMappingForDecl(loc logger.Loc) {
-	if !p.hasDecl {
-		p.hasDecl = true
-		p.firstDeclByteOffset = uint32(len(p.js))
-		p.firstDeclSourceMapOffset = uint32(len(p.sourceMap))
-		p.prevLoc.Start = -1 // Force a new source mapping to be generated
-		p.addSourceMapping(loc)
-	}
-}
-
 func (p *printer) printBinding(binding js_ast.Binding) {
+	p.addSourceMapping(binding.Loc)
+
 	switch b := binding.Data.(type) {
 	case *js_ast.BMissing:
 
 	case *js_ast.BIdentifier:
-		p.printSpaceBeforeIdentifier()
-		p.addSourceMappingForDecl(binding.Loc)
 		p.printSymbol(b.Ref)
 
 	case *js_ast.BArray:
-		p.addSourceMappingForDecl(binding.Loc)
 		p.print("[")
 		if len(b.Items) > 0 {
 			if !b.IsSingleLine {
@@ -933,7 +863,6 @@ func (p *printer) printBinding(binding js_ast.Binding) {
 		p.print("]")
 
 	case *js_ast.BObject:
-		p.addSourceMappingForDecl(binding.Loc)
 		p.print("{")
 		if len(b.Properties) > 0 {
 			if !b.IsSingleLine {
@@ -2010,7 +1939,7 @@ func (p *printer) printExpr(expr js_ast.Expr, level js_ast.L, flags int) {
 		p.printSpaceBeforeIdentifier()
 		p.print("await")
 		p.printSpace()
-		p.printExpr(e.Value, js_ast.LPrefix, 0)
+		p.printExpr(e.Value, js_ast.LPrefix-1, 0)
 
 		if wrap {
 			p.print(")")
@@ -2109,6 +2038,8 @@ func (p *printer) printExpr(expr js_ast.Expr, level js_ast.L, flags int) {
 			// "**" can't contain certain unary expressions
 			if left, ok := e.Left.Data.(*js_ast.EUnary); ok && left.Op.UnaryAssignTarget() == js_ast.AssignTargetNone {
 				leftLevel = js_ast.LCall
+			} else if _, ok := e.Left.Data.(*js_ast.EAwait); ok {
+				leftLevel = js_ast.LCall
 			} else if _, ok := e.Left.Data.(*js_ast.EUndefined); ok {
 				// Undefined is printed as "void 0"
 				leftLevel = js_ast.LCall
@@ -2333,8 +2264,6 @@ func (p *printer) printDeclStmt(isExport bool, keyword string, decls []js_ast.De
 		p.print("export ")
 	}
 	p.printDecls(keyword, decls, 0)
-	p.updateGeneratedLineAndColumn()
-	p.finalLocalSemi = p.generatedColumn
 	p.printSemicolonAfterStatement()
 }
 
@@ -3105,10 +3034,6 @@ type SourceMapChunk struct {
 	// we'll need to know how many characters were in the last line we generated.
 	FinalGeneratedColumn int
 
-	// Like "FinalGeneratedColumn" but for the generated column of the last
-	// semicolon for a "SLocal" statement.
-	FinalLocalSemi int
-
 	ShouldIgnore bool
 }
 
@@ -3121,10 +3046,6 @@ type PrintResult struct {
 	SourceMapChunk SourceMapChunk
 
 	ExtractedComments map[string]bool
-
-	// These are used when stripping off the leading variable declaration
-	FirstDeclByteOffset      uint32
-	FirstDeclSourceMapOffset uint32
 }
 
 func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options Options) PrintResult {
@@ -3172,10 +3093,7 @@ func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options
 			Buffer:               p.sourceMap,
 			EndState:             p.prevState,
 			FinalGeneratedColumn: p.generatedColumn,
-			FinalLocalSemi:       p.finalLocalSemi,
 			ShouldIgnore:         p.shouldIgnoreSourceMap(),
 		},
-		FirstDeclByteOffset:      p.firstDeclByteOffset,
-		FirstDeclSourceMapOffset: p.firstDeclSourceMapOffset,
 	}
 }
