@@ -15,6 +15,8 @@ import (
 	"time"
 )
 
+const defaultTerminalWidth = 80
+
 type Log struct {
 	AddMsg    func(Msg)
 	HasErrors func() bool
@@ -61,6 +63,9 @@ type Msg struct {
 type MsgData struct {
 	Text     string
 	Location *MsgLocation
+
+	// Optional user-specified data that is passed through unmodified
+	UserDetail interface{}
 }
 
 type MsgLocation struct {
@@ -311,11 +316,13 @@ func NewStderrLog(options OutputOptions) Log {
 			case Error:
 				errors++
 				if options.LogLevel <= LevelError {
+					options.MaybeWriteFirstNewline(os.Stderr)
 					writeStringWithColor(os.Stderr, msg.String(options, terminalInfo))
 				}
 			case Warning:
 				warnings++
 				if options.LogLevel <= LevelWarning {
+					options.MaybeWriteFirstNewline(os.Stderr)
 					writeStringWithColor(os.Stderr, msg.String(options, terminalInfo))
 				}
 			}
@@ -324,6 +331,7 @@ func NewStderrLog(options OutputOptions) Log {
 			if options.ErrorLimit != 0 && errors >= options.ErrorLimit {
 				errorLimitWasHit = true
 				if options.LogLevel <= LevelError {
+					options.MaybeWriteFirstNewline(os.Stderr)
 					writeStringWithColor(os.Stderr, fmt.Sprintf(
 						"%s reached (disable error limit with --error-limit=0)\n", errorAndWarningSummary(errors, warnings)))
 				}
@@ -340,6 +348,7 @@ func NewStderrLog(options OutputOptions) Log {
 
 			// Print out a summary if the error limit wasn't hit
 			if !errorLimitWasHit && options.LogLevel <= LevelInfo && (warnings != 0 || errors != 0) {
+				options.MaybeWriteFirstNewline(os.Stderr)
 				writeStringWithColor(os.Stderr, fmt.Sprintf("%s\n", errorAndWarningSummary(errors, warnings)))
 			}
 
@@ -534,7 +543,11 @@ func PrintSummary(osArgs []string, table SummaryTable, start time.Time) {
 			}
 
 			margin := "  "
-			layoutWidth := GetTerminalInfo(os.Stderr).Width - 2*len(margin)
+			layoutWidth := GetTerminalInfo(os.Stderr).Width
+			if layoutWidth < 1 {
+				layoutWidth = defaultTerminalWidth
+			}
+			layoutWidth -= 2 * len(margin)
 			if hasSizeWarning {
 				// Add space for the warning icon
 				layoutWidth -= 2
@@ -684,6 +697,16 @@ type OutputOptions struct {
 	ErrorLimit    int
 	Color         UseColor
 	LogLevel      LogLevel
+
+	// This is used with watch mode
+	NewlineBeforeFirstMessage bool
+}
+
+func (opts *OutputOptions) MaybeWriteFirstNewline(file *os.File) {
+	if opts.NewlineBeforeFirstMessage {
+		opts.NewlineBeforeFirstMessage = false
+		file.WriteString("\n")
+	}
 }
 
 func (msg Msg) String(options OutputOptions, terminalInfo TerminalInfo) string {
@@ -750,7 +773,7 @@ func msgString(options OutputOptions, terminalInfo TerminalInfo, kind MsgKind, d
 		kindColor = colorResetBold
 		textResetColor = colorReset
 		if options.IncludeSource {
-			textIndent = strings.Repeat(" ", maxMargin+extraMarginChars-3)
+			textIndent = "   "
 		}
 
 	default:
@@ -927,7 +950,7 @@ func detailStruct(data MsgData, terminalInfo TerminalInfo, maxMargin int) MsgDet
 	// Trim the line to fit the terminal width
 	width := terminalInfo.Width
 	if width < 1 {
-		width = 80
+		width = defaultTerminalWidth
 	}
 	width -= maxMargin + extraMarginChars
 	if width < 1 {
