@@ -30,7 +30,7 @@ var defaultMainFields = map[config.Platform][]string{
 	// This is deliberate because the presence of the "browser" field is a
 	// good signal that the "module" field may have non-browser stuff in it,
 	// which will crash or fail to be bundled when targeting the browser.
-	config.PlatformBrowser: []string{"browser", "module", "main"},
+	config.PlatformBrowser: {"browser", "module", "main"},
 
 	// Note that this means if a package specifies "module" and "main", the ES6
 	// module will not be selected. This means tree shaking will not work when
@@ -48,7 +48,12 @@ var defaultMainFields = map[config.Platform][]string{
 	// If you want to enable tree shaking when targeting node, you will have to
 	// configure the main fields to be "module" and then "main". Keep in mind
 	// that some packages may break if you do this.
-	config.PlatformNode: []string{"main", "module"},
+	config.PlatformNode: {"main", "module"},
+
+	// The neutral platform is for people that don't want esbuild to try to
+	// pick good defaults for their platform. In that case, the list of main
+	// fields is empty by default. You must explicitly configure it yourself.
+	config.PlatformNeutral: {},
 }
 
 // Path resolution is a mess. One tricky issue is the "module" override for the
@@ -87,6 +92,9 @@ type IgnoreIfUnusedData struct {
 type ResolveResult struct {
 	PathPair PathPair
 
+	// If this was resolved by a plugin, the plugin gets to store its data here
+	PluginData interface{}
+
 	// If not empty, these should override the default values
 	JSXFactory  []string // Default if empty: "React.createElement"
 	JSXFragment []string // Default if empty: "React.Fragment"
@@ -104,9 +112,6 @@ type ResolveResult struct {
 	// behavior of the "importsNotUsedAsValues" field in "tsconfig.json" when the
 	// value is not "remove".
 	PreserveUnusedImportsTS bool
-
-	// This is true if the file is inside a "node_modules" directory
-	SuppressWarningsAboutWeirdCode bool
 }
 
 type Resolver interface {
@@ -263,17 +268,22 @@ func (r *resolver) ProbeResolvePackageAsRelative(sourceDir string, importPath st
 	return nil
 }
 
-func isInsideNodeModules(fs fs.FS, path string) bool {
-	dir := fs.Dir(path)
+func IsInsideNodeModules(fs fs.FS, path string) bool {
 	for {
-		if fs.Base(dir) == "node_modules" {
-			return true
-		}
-		parent := fs.Dir(dir)
-		if dir == parent {
+		// This is written in a platform-independent manner because it's run on
+		// user-specified paths which can be arbitrary non-file-system things. So
+		// for example Windows paths may end up being used on Unix or URLs may end
+		// up being used on Windows. Be consistently agnostic to which kind of
+		// slash is used on all platforms.
+		slash := strings.LastIndexAny(path, "/\\")
+		if slash == -1 {
 			return false
 		}
-		dir = parent
+		dir, base := path[:slash], path[slash+1:]
+		if base == "node_modules" {
+			return true
+		}
+		path = dir
 	}
 }
 
@@ -283,11 +293,6 @@ func (r *resolver) finalizeResolve(result ResolveResult) *ResolveResult {
 			if dirInfo := r.dirInfoCached(r.fs.Dir(path.Text)); dirInfo != nil {
 				base := r.fs.Base(path.Text)
 
-				// Don't emit warnings for code inside a "node_modules" directory
-				if isInsideNodeModules(r.fs, path.Text) {
-					result.SuppressWarningsAboutWeirdCode = true
-				}
-
 				// Look up this file in the "sideEffects" map in the nearest enclosing
 				// directory with a "package.json" file.
 				//
@@ -332,7 +337,7 @@ func (r *resolver) finalizeResolve(result ResolveResult) *ResolveResult {
 				}
 
 				if entry, ok := dirInfo.entries[base]; ok {
-					if symlink := entry.Symlink(); symlink != "" {
+					if symlink := entry.Symlink(r.fs); symlink != "" {
 						// Is this entry itself a symlink?
 						path.Text = symlink
 					} else if dirInfo.absRealPath != "" {
@@ -668,7 +673,7 @@ func (r *resolver) parseTSConfig(file string, visited map[string]bool) (*TSConfi
 		}
 
 		// Suppress warnings about missing base config files inside "node_modules"
-		if !isInsideNodeModules(r.fs, file) {
+		if !IsInsideNodeModules(r.fs, file) {
 			r.log.AddRangeWarning(&source, extendsRange,
 				fmt.Sprintf("Cannot find base config file %q", extends))
 		}
@@ -724,7 +729,7 @@ func (r *resolver) dirInfoUncached(path string) *dirInfo {
 	base := r.fs.Base(path)
 	if base != "node_modules" {
 		if entry, ok := entries["node_modules"]; ok {
-			info.hasNodeModules = entry.Kind() == fs.DirEntry
+			info.hasNodeModules = entry.Kind(r.fs) == fs.DirEntry
 		}
 	}
 
@@ -734,7 +739,7 @@ func (r *resolver) dirInfoUncached(path string) *dirInfo {
 
 		// Make sure "absRealPath" is the real path of the directory (resolving any symlinks)
 		if entry, ok := parentInfo.entries[base]; ok {
-			if symlink := entry.Symlink(); symlink != "" {
+			if symlink := entry.Symlink(r.fs); symlink != "" {
 				info.absRealPath = symlink
 			} else if parentInfo.absRealPath != "" {
 				info.absRealPath = r.fs.Join(parentInfo.absRealPath, base)
@@ -743,7 +748,7 @@ func (r *resolver) dirInfoUncached(path string) *dirInfo {
 	}
 
 	// Record if this directory has a package.json file
-	if entry, ok := entries["package.json"]; ok && entry.Kind() == fs.FileEntry {
+	if entry, ok := entries["package.json"]; ok && entry.Kind(r.fs) == fs.FileEntry {
 		info.packageJSON = r.parsePackageJSON(path)
 
 		// Propagate this browser scope into child directories
@@ -756,9 +761,9 @@ func (r *resolver) dirInfoUncached(path string) *dirInfo {
 	{
 		var tsConfigPath string
 		if forceTsConfig := r.options.TsConfigOverride; forceTsConfig == "" {
-			if entry, ok := entries["tsconfig.json"]; ok && entry.Kind() == fs.FileEntry {
+			if entry, ok := entries["tsconfig.json"]; ok && entry.Kind(r.fs) == fs.FileEntry {
 				tsConfigPath = r.fs.Join(path, "tsconfig.json")
-			} else if entry, ok := entries["jsconfig.json"]; ok && entry.Kind() == fs.FileEntry {
+			} else if entry, ok := entries["jsconfig.json"]; ok && entry.Kind(r.fs) == fs.FileEntry {
 				tsConfigPath = r.fs.Join(path, "jsconfig.json")
 			}
 		} else if parentInfo == nil {
@@ -1005,13 +1010,13 @@ func (r *resolver) loadAsFile(path string, extensionOrder []string) (string, boo
 	base := r.fs.Base(path)
 
 	// Try the plain path without any extensions
-	if entry, ok := entries[base]; ok && entry.Kind() == fs.FileEntry {
+	if entry, ok := entries[base]; ok && entry.Kind(r.fs) == fs.FileEntry {
 		return path, true
 	}
 
 	// Try the path with extensions
 	for _, ext := range extensionOrder {
-		if entry, ok := entries[base+ext]; ok && entry.Kind() == fs.FileEntry {
+		if entry, ok := entries[base+ext]; ok && entry.Kind(r.fs) == fs.FileEntry {
 			return path + ext, true
 		}
 	}
@@ -1036,7 +1041,7 @@ func (r *resolver) loadAsFile(path string, extensionOrder []string) (string, boo
 		// Note that the official compiler code always tries ".ts" before
 		// ".tsx" even if the original extension was ".jsx".
 		for _, ext := range []string{".ts", ".tsx"} {
-			if entry, ok := entries[base[:lastDot]+ext]; ok && entry.Kind() == fs.FileEntry {
+			if entry, ok := entries[base[:lastDot]+ext]; ok && entry.Kind(r.fs) == fs.FileEntry {
 				return path[:len(path)-(len(base)-lastDot)] + ext, true
 			}
 		}
@@ -1052,7 +1057,7 @@ func (r *resolver) loadAsIndex(path string, entries map[string]*fs.Entry) (strin
 	// Try the "index" file with extensions
 	for _, ext := range r.options.ExtensionOrder {
 		base := "index" + ext
-		if entry, ok := entries[base]; ok && entry.Kind() == fs.FileEntry {
+		if entry, ok := entries[base]; ok && entry.Kind(r.fs) == fs.FileEntry {
 			return r.fs.Join(path, base), true
 		}
 	}
