@@ -84,6 +84,7 @@ const (
 	SourceMapInline
 	SourceMapLinked
 	SourceMapExternal
+	SourceMapInlineAndExternal
 )
 
 type SourcesContent uint8
@@ -129,6 +130,7 @@ type Platform uint8
 const (
 	PlatformBrowser Platform = iota
 	PlatformNode
+	PlatformNeutral
 )
 
 type Format uint8
@@ -168,6 +170,10 @@ type Location struct {
 type Message struct {
 	Text     string
 	Location *Location
+
+	// Optional user-specified data that is passed through unmodified. You can
+	// use this to stash the original error, for example.
+	Detail interface{}
 }
 
 type StderrColor uint8
@@ -237,6 +243,7 @@ type BuildOptions struct {
 	Metafile          string
 	Outdir            string
 	Outbase           string
+	AbsWorkingDir     string
 	Platform          Platform
 	Format            Format
 	External          []string
@@ -255,6 +262,14 @@ type BuildOptions struct {
 	Write       bool
 	Incremental bool
 	Plugins     []Plugin
+
+	Watch *WatchMode
+}
+
+type WatchMode struct {
+	SpinnerBusy string
+	SpinnerIdle []string
+	OnRebuild   func(BuildResult)
 }
 
 type StdinOptions struct {
@@ -270,7 +285,8 @@ type BuildResult struct {
 
 	OutputFiles []OutputFile
 
-	Rebuild func() BuildResult
+	Rebuild func() BuildResult // Only when "Incremental: true"
+	Stop    func()             // Only when "Watch: true"
 }
 
 type OutputFile struct {
@@ -279,7 +295,7 @@ type OutputFile struct {
 }
 
 func Build(options BuildOptions) BuildResult {
-	return buildImpl(options)
+	return buildImpl(options).result
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -382,6 +398,7 @@ type OnResolveArgs struct {
 	Importer   string
 	Namespace  string
 	ResolveDir string
+	PluginData interface{}
 }
 
 type OnResolveResult struct {
@@ -390,9 +407,10 @@ type OnResolveResult struct {
 	Errors   []Message
 	Warnings []Message
 
-	Path      string
-	External  bool
-	Namespace string
+	Path       string
+	External   bool
+	Namespace  string
+	PluginData interface{}
 }
 
 type OnLoadOptions struct {
@@ -401,8 +419,9 @@ type OnLoadOptions struct {
 }
 
 type OnLoadArgs struct {
-	Path      string
-	Namespace string
+	Path       string
+	Namespace  string
+	PluginData interface{}
 }
 
 type OnLoadResult struct {
@@ -414,4 +433,5 @@ type OnLoadResult struct {
 	Contents   *string
 	ResolveDir string
 	Loader     Loader
+	PluginData interface{}
 }
