@@ -5,13 +5,13 @@ import (
 	"io/ioutil"
 	"net"
 	"os"
-	"path/filepath"
 	"sort"
 	"strconv"
 	"strings"
 	"time"
 
 	"github.com/evanw/esbuild/internal/cli_helpers"
+	"github.com/evanw/esbuild/internal/fs"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/pkg/api"
 )
@@ -41,6 +41,9 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 		case arg == "--splitting" && buildOpts != nil:
 			buildOpts.Splitting = true
 
+		case arg == "--watch" && buildOpts != nil:
+			buildOpts.Watch = &api.WatchMode{}
+
 		case arg == "--minify":
 			if buildOpts != nil {
 				buildOpts.MinifySyntax = true
@@ -127,19 +130,23 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 			}
 			hasBareSourceMapFlag = true
 
-		case arg == "--sourcemap=external":
-			if buildOpts != nil {
-				buildOpts.Sourcemap = api.SourceMapExternal
-			} else {
-				transformOpts.Sourcemap = api.SourceMapExternal
+		case strings.HasPrefix(arg, "--sourcemap="):
+			value := arg[len("--sourcemap="):]
+			var sourcemap api.SourceMap
+			switch value {
+			case "inline":
+				sourcemap = api.SourceMapInline
+			case "external":
+				sourcemap = api.SourceMapExternal
+			case "both":
+				sourcemap = api.SourceMapInlineAndExternal
+			default:
+				return fmt.Errorf("Invalid sourcemap: %q (valid: inline, external, both)", value)
 			}
-			hasBareSourceMapFlag = false
-
-		case arg == "--sourcemap=inline":
 			if buildOpts != nil {
-				buildOpts.Sourcemap = api.SourceMapInline
+				buildOpts.Sourcemap = sourcemap
 			} else {
-				transformOpts.Sourcemap = api.SourceMapInline
+				transformOpts.Sourcemap = sourcemap
 			}
 			hasBareSourceMapFlag = false
 
@@ -286,8 +293,10 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				buildOpts.Platform = api.PlatformBrowser
 			case "node":
 				buildOpts.Platform = api.PlatformNode
+			case "neutral":
+				buildOpts.Platform = api.PlatformNeutral
 			default:
-				return fmt.Errorf("Invalid platform: %q (valid: browser, node)", value)
+				return fmt.Errorf("Invalid platform: %q (valid: browser, node, neutral)", value)
 			}
 
 		case strings.HasPrefix(arg, "--format="):
@@ -553,6 +562,19 @@ func runImpl(osArgs []string) int {
 
 	switch {
 	case buildOptions != nil:
+		// These characters were chosen because they work in Windows Command Prompt
+		if buildOptions.Watch != nil {
+			buildOptions.Watch.SpinnerBusy = "··· "
+			buildOptions.Watch.SpinnerIdle = []string{
+				"▫▫▫ ",
+				"▪▫▫ ",
+				"▪▪▫ ",
+				"▪▪▪ ",
+				"▫▪▪ ",
+				"▫▫▪ ",
+			}
+		}
+
 		// Read from stdin when there are no entry points
 		if len(buildOptions.EntryPoints) == 0 {
 			if buildOptions.Stdin == nil {
@@ -583,6 +605,11 @@ func runImpl(osArgs []string) int {
 			return 1
 		}
 
+		// Do not exit if we're in watch mode
+		if buildOptions.Watch != nil {
+			<-make(chan bool)
+		}
+
 		// Print a summary to stderr
 		if shouldPrintSummary {
 			printSummary(osArgs, result.OutputFiles, start)
@@ -623,31 +650,33 @@ func printSummary(osArgs []string, outputFiles []api.OutputFile, start time.Time
 	var table logger.SummaryTable = make([]logger.SummaryTableEntry, len(outputFiles))
 
 	if len(outputFiles) > 0 {
-		cwd, _ := os.Getwd()
-
-		for i, file := range outputFiles {
-			path, err := filepath.Rel(cwd, file.Path)
-			if err != nil {
-				path = file.Path
-			}
-			dir, base := filepath.Split(path)
-			n := len(file.Contents)
-			var size string
-			if n < 1024 {
-				size = fmt.Sprintf("%db ", n)
-			} else if n < 1024*1024 {
-				size = fmt.Sprintf("%.1fkb", float64(n)/(1024))
-			} else if n < 1024*1024*1024 {
-				size = fmt.Sprintf("%.1fmb", float64(n)/(1024*1024))
-			} else {
-				size = fmt.Sprintf("%.1fgb", float64(n)/(1024*1024*1024))
-			}
-			table[i] = logger.SummaryTableEntry{
-				Dir:         dir,
-				Base:        base,
-				Size:        size,
-				Bytes:       n,
-				IsSourceMap: strings.HasSuffix(base, ".map"),
+		if cwd, err := os.Getwd(); err == nil {
+			if realFS, err := fs.RealFS(fs.RealFSOptions{AbsWorkingDir: cwd}); err == nil {
+				for i, file := range outputFiles {
+					path, ok := realFS.Rel(realFS.Cwd(), file.Path)
+					if !ok {
+						path = file.Path
+					}
+					base := realFS.Base(path)
+					n := len(file.Contents)
+					var size string
+					if n < 1024 {
+						size = fmt.Sprintf("%db ", n)
+					} else if n < 1024*1024 {
+						size = fmt.Sprintf("%.1fkb", float64(n)/(1024))
+					} else if n < 1024*1024*1024 {
+						size = fmt.Sprintf("%.1fmb", float64(n)/(1024*1024))
+					} else {
+						size = fmt.Sprintf("%.1fgb", float64(n)/(1024*1024*1024))
+					}
+					table[i] = logger.SummaryTableEntry{
+						Dir:         path[:len(path)-len(base)],
+						Base:        base,
+						Size:        size,
+						Bytes:       n,
+						IsSourceMap: strings.HasSuffix(base, ".map"),
+					}
+				}
 			}
 		}
 	}
