@@ -1,6 +1,7 @@
 const childProcess = require('child_process')
 const rimraf = require('rimraf')
 const path = require('path')
+const zlib = require('zlib')
 const fs = require('fs')
 const os = require('os')
 
@@ -8,7 +9,7 @@ const repoDir = path.dirname(__dirname)
 const npmDir = path.join(repoDir, 'npm', 'esbuild')
 const version = fs.readFileSync(path.join(repoDir, 'version.txt'), 'utf8').trim()
 
-function buildNativeLib(esbuildPath) {
+exports.buildNativeLib = (esbuildPath) => {
   const libDir = path.join(npmDir, 'lib')
   fs.mkdirSync(libDir, { recursive: true })
 
@@ -38,13 +39,53 @@ function buildNativeLib(esbuildPath) {
   fs.writeFileSync(path.join(libDir, 'main.d.ts'), types_ts)
 }
 
-function buildWasmLib(esbuildPath) {
+exports.buildWasmLib = async (esbuildPath) => {
+  // Asynchronously start building the WebAssembly module
   const npmWasmDir = path.join(repoDir, 'npm', 'esbuild-wasm')
+  const goBuildPromise = new Promise((resolve, reject) => childProcess.execFile('go',
+    ['build', '-o', path.join(npmWasmDir, 'esbuild.wasm'), path.join(repoDir, 'cmd', 'esbuild')],
+    { cwd: repoDir, stdio: 'inherit', env: { ...process.env, GOOS: 'js', GOARCH: 'wasm' } },
+    err => err ? reject(err) : resolve()))
+
   const libDir = path.join(npmWasmDir, 'lib')
   const esmDir = path.join(npmWasmDir, 'esm')
   fs.mkdirSync(libDir, { recursive: true })
   fs.mkdirSync(esmDir, { recursive: true })
 
+  // Generate "npm/esbuild-wasm/wasm_exec.js"
+  const toReplace = 'global.fs = fs;';
+  const GOROOT = childProcess.execFileSync('go', ['env', 'GOROOT']).toString().trim();
+  let wasm_exec_js = fs.readFileSync(path.join(GOROOT, 'misc', 'wasm', 'wasm_exec.js'), 'utf8');
+  let index = wasm_exec_js.indexOf(toReplace);
+  if (index === -1) throw new Error(`Failed to find ${JSON.stringify(toReplace)} in Go JS shim code`);
+  wasm_exec_js = wasm_exec_js.replace(toReplace, `
+    global.fs = Object.assign({}, fs, {
+      // Hack around a Unicode bug in node: https://github.com/nodejs/node/issues/24550
+      write(fd, buf, offset, length, position, callback) {
+        if (offset === 0 && length === buf.length && position === null) {
+          if (fd === process.stdout.fd) {
+            try {
+              process.stdout.write(buf, err => err ? callback(err, 0, null) : callback(null, length, buf));
+            } catch (err) {
+              callback(err, 0, null);
+            }
+            return;
+          }
+          if (fd === process.stderr.fd) {
+            try {
+              process.stderr.write(buf, err => err ? callback(err, 0, null) : callback(null, length, buf));
+            } catch (err) {
+              callback(err, 0, null);
+            }
+            return;
+          }
+        }
+        fs.write(fd, buf, offset, length, position, callback);
+      },
+    });
+  `);
+  fs.writeFileSync(path.join(npmWasmDir, 'wasm_exec.js'), wasm_exec_js);
+
   // Generate "npm/esbuild-wasm/lib/main.js"
   childProcess.execFileSync(esbuildPath, [
     path.join(repoDir, 'lib', 'node.ts'),
@@ -63,48 +104,75 @@ function buildWasmLib(esbuildPath) {
   fs.writeFileSync(path.join(libDir, 'browser.d.ts'), types_ts)
   fs.writeFileSync(path.join(esmDir, 'browser.d.ts'), types_ts)
 
-  // Minify "npm/esbuild-wasm/wasm_exec.js"
-  const wasm_exec_js = path.join(npmWasmDir, 'wasm_exec.js')
-  const wasmExecMin = childProcess.execFileSync(esbuildPath, [
-    wasm_exec_js,
-    '--minify',
-  ], { cwd: repoDir }).toString()
-  const commentLines = fs.readFileSync(wasm_exec_js, 'utf8').split('\n')
-  const firstNonComment = commentLines.findIndex(line => !line.startsWith('//'))
-  const wasmExecMinCode = '\n' + commentLines.slice(0, firstNonComment).concat(wasmExecMin).join('\n')
+  for (const minify of [false, true]) {
+    const minifyFlags = minify ? ['--minify'] : []
 
-  // Minify "lib/worker.ts"
-  const workerMinCode = childProcess.execFileSync(esbuildPath, [
-    path.join(repoDir, 'lib', 'worker.ts'),
-    '--minify',
-    '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-  ], { cwd: repoDir }).toString().trim()
+    // Process "npm/esbuild-wasm/wasm_exec.js"
+    let wasmExecCode = wasm_exec_js;
+    if (minify) {
+      const wasmExecMin = childProcess.execFileSync(esbuildPath, [
+        '--target=es2015',
+      ].concat(minifyFlags), { cwd: repoDir, input: wasmExecCode }).toString()
+      const commentLines = wasmExecCode.split('\n')
+      const firstNonComment = commentLines.findIndex(line => !line.startsWith('//'))
+      wasmExecCode = '\n' + commentLines.slice(0, firstNonComment).concat(wasmExecMin).join('\n')
+    }
 
-  // Generate "npm/esbuild-wasm/lib/browser.js"
-  const umdPrefix = `(exports=>{`
-  const umdSuffix = `})(typeof exports==="object"?exports:(typeof self!=="undefined"?self:this).esbuild={});\n`
-  const browserCJS = childProcess.execFileSync(esbuildPath, [
-    path.join(repoDir, 'lib', 'browser.ts'),
-    '--bundle',
-    '--target=es2015',
-    '--minify',
-    '--format=cjs',
-    '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-    '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmExecMinCode + workerMinCode),
-  ], { cwd: repoDir }).toString()
-  fs.writeFileSync(path.join(libDir, 'browser.js'), umdPrefix + browserCJS.trim() + umdSuffix)
+    // Process "lib/worker.ts"
+    const workerCode = childProcess.execFileSync(esbuildPath, [
+      path.join(repoDir, 'lib', 'worker.ts'),
+      '--target=es2015',
+      '--define:ESBUILD_VERSION=' + JSON.stringify(version),
+    ].concat(minifyFlags), { cwd: repoDir }).toString().trim()
 
-  // Generate "npm/esbuild-wasm/esm/browser.js"
-  const browserESM = childProcess.execFileSync(esbuildPath, [
-    path.join(repoDir, 'lib', 'browser.ts'),
-    '--bundle',
-    '--target=es2017',
-    '--minify',
-    '--format=esm',
-    '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-    '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmExecMinCode + workerMinCode),
-  ], { cwd: repoDir }).toString()
-  fs.writeFileSync(path.join(esmDir, 'browser.js'), browserESM.trim())
+    // Generate "npm/esbuild-wasm/lib/browser.*"
+    const umdPrefix = `(exports=>{`
+    const umdSuffix = `})(typeof exports==="object"?exports:(typeof self!=="undefined"?self:this).esbuild={});`
+    const browserCJS = childProcess.execFileSync(esbuildPath, [
+      path.join(repoDir, 'lib', 'browser.ts'),
+      '--bundle',
+      '--target=es2015',
+      '--format=cjs',
+      '--define:ESBUILD_VERSION=' + JSON.stringify(version),
+      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmExecCode + workerCode),
+      '--banner=' + umdPrefix,
+      '--footer=' + umdSuffix,
+    ].concat(minifyFlags), { cwd: repoDir }).toString()
+    fs.writeFileSync(path.join(libDir, minify ? 'browser.min.js' : 'browser.js'), browserCJS)
+
+    // Generate "npm/esbuild-wasm/esm/browser.min.js"
+    const browserESM = childProcess.execFileSync(esbuildPath, [
+      path.join(repoDir, 'lib', 'browser.ts'),
+      '--bundle',
+      '--target=es2017',
+      '--format=esm',
+      '--define:ESBUILD_VERSION=' + JSON.stringify(version),
+      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmExecCode + workerCode),
+    ].concat(minifyFlags), { cwd: repoDir }).toString()
+    fs.writeFileSync(path.join(esmDir, minify ? 'browser.min.js' : 'browser.js'), browserESM)
+  }
+
+  // Generate the "exit0" stubs
+  const exit0Map = {};
+  const exit0Dir = path.join(__dirname, '..', 'lib', 'exit0');
+  for (const entry of fs.readdirSync(exit0Dir)) {
+    if (entry.endsWith('.node')) {
+      const absPath = path.join(exit0Dir, entry);
+      const compressed = zlib.deflateRawSync(fs.readFileSync(absPath), { level: 9 });
+      exit0Map[entry] = compressed.toString('base64');
+    }
+  }
+  fs.writeFileSync(path.join(npmWasmDir, 'exit0.js'), `
+// Each of these is a native module that calls "exit(0)". This is a workaround
+// for https://github.com/nodejs/node/issues/36616. These native modules are
+// stored in a string both to make them smaller and to hide them from Yarn 2,
+// since they make Yarn 2 unzip this package.
+
+module.exports = ${JSON.stringify(exit0Map, null, 2)};
+`);
+
+  // Join with the asynchronous WebAssembly build
+  await goBuildPromise;
 }
 
 exports.buildBinary = () => {
@@ -135,7 +203,7 @@ exports.removeRecursiveSync = path => {
 exports.installForTests = () => {
   // Build the "esbuild" binary and library
   const esbuildPath = exports.buildBinary()
-  buildNativeLib(esbuildPath)
+  exports.buildNativeLib(esbuildPath)
 
   // Install the "esbuild" package to a temporary directory. On Windows, it's
   // sometimes randomly impossible to delete this installation directory. My
@@ -165,8 +233,8 @@ exports.dirname = __dirname
 // The main Makefile invokes this script before publishing
 if (require.main === module) {
   if (process.argv.indexOf('--wasm') >= 0) {
-    buildWasmLib(process.argv[2])
+    exports.buildWasmLib(process.argv[2])
   } else {
-    buildNativeLib(process.argv[2])
+    exports.buildNativeLib(process.argv[2])
   }
 }
