@@ -1,5 +1,5 @@
+const { removeRecursiveSync } = require('./esbuild')
 const child_process = require('child_process')
-const rimraf = require('rimraf')
 const path = require('path')
 const fs = require('fs')
 
@@ -43,6 +43,103 @@ const tests = {
     esbuild.startService().then(service => service.transform(''))
     esbuild.startService().then(service => service.transform('', {}))
   `,
+  writeFalseOutputFiles: `
+    import * as esbuild from 'esbuild'
+    esbuild.buildSync({ write: false }).outputFiles[0]
+    esbuild.build({ write: false }).then(result => result.outputFiles[0])
+    esbuild.startService().then(service => service.build({ write: false }).then(result => result.outputFiles[0]))
+  `,
+  incrementalTrueRebuild: `
+    import * as esbuild from 'esbuild'
+    esbuild.build({ incremental: true }).then(result => {
+      result.rebuild().then(result => {
+        result.rebuild().then(() => {
+          result.rebuild.dispose()
+        })
+      })
+      result.rebuild.dispose()
+    })
+    esbuild.startService().then(service => {
+      service.build({ incremental: true }).then(result => {
+        result.rebuild().then(result => {
+          result.rebuild().then(() => {
+            result.rebuild.dispose()
+          })
+        })
+        result.rebuild.dispose()
+      })
+    })
+    async function a() {
+      let result = await esbuild.build({ incremental: true })
+      let result2 = await result.rebuild()
+      await result2.rebuild()
+      result2.rebuild.dispose()
+      result.rebuild.dispose()
+    }
+    async function b() {
+      let service = await esbuild.startService()
+      let result = await service.build({ incremental: true })
+      let result2 = await result.rebuild()
+      await result2.rebuild()
+      result2.rebuild.dispose()
+      result.rebuild.dispose()
+      service.stop()
+    }
+  `,
+  ifRebuild: `
+    import * as esbuild from 'esbuild'
+    let options: any
+    esbuild.build(options).then(result => {
+      if (result.rebuild) {
+        result.rebuild().then(result => {
+          if (result.rebuild) {
+            result.rebuild().then(result => {
+              result.rebuild.dispose()
+            })
+          }
+          result.rebuild.dispose()
+        })
+      }
+    })
+    esbuild.startService().then(service => {
+      service.build(options).then(result => {
+        if (result.rebuild) {
+          result.rebuild().then(result => {
+            if (result.rebuild) {
+              result.rebuild().then(result => {
+                result.rebuild.dispose()
+              })
+            }
+            result.rebuild.dispose()
+          })
+        }
+      })
+    })
+    async function a() {
+      let result = await esbuild.build(options)
+      if (result.rebuild) {
+        let result2 = await result.rebuild()
+        if (result2.rebuild) {
+          await result2.rebuild()
+          result2.rebuild.dispose()
+        }
+        result.rebuild.dispose()
+      }
+    }
+    async function b() {
+      let service = await esbuild.startService()
+      let result = await service.build(options)
+      if (result.rebuild) {
+        let result2 = await result.rebuild()
+        if (result2.rebuild) {
+          await result2.rebuild()
+          result2.rebuild.dispose()
+        }
+        result.rebuild.dispose()
+      }
+      service.stop()
+    }
+  `,
   allOptionsTransform: `
     export {}
     import {transform} from 'esbuild'
@@ -221,7 +318,7 @@ const tests = {
 
 async function main() {
   let testDir = path.join(__dirname, '.ts-types-test')
-  rimraf.sync(testDir, { disableGlob: true })
+  removeRecursiveSync(testDir)
   fs.mkdirSync(testDir, { recursive: true })
   fs.writeFileSync(path.join(testDir, 'tsconfig.json'), JSON.stringify(tsconfigJson))
 
@@ -252,7 +349,7 @@ async function main() {
     process.exit(1)
   } else {
     console.log(`✅ typescript type tests passed`)
-    rimraf.sync(testDir, { disableGlob: true })
+    removeRecursiveSync(testDir)
   }
 }
 
